#!/usr/bin/env python
# encoding: utf-8
'''
@license: (C) Copyright 2021, Hey.
@author: Hey
@email: sanyuan.hy@alibaba-inc.com
@tel: 137****6540
@datetime: 2023/4/21 20:57
@project: LucaProtApp
@file: inference_time
@desc: xxxx
'''

import time, sys
import os, csv
import subprocess
import argparse
sys.path.append(".")
sys.path.append("..")
sys.path.append("../algorithms")
try:
    from utils import write_fasta
except ImportError:
    from algorithms.utils import write_fasta

parser = argparse.ArgumentParser(description="Prediction")
parser.add_argument("--group_idx", default=-1, type=int, help="only running the group_idx not greater than this value")
parser.add_argument("--method", default=None, choices=["diamond", "hmmscan", "palmscan"], type=str, help="the classic method")
parser.add_argument("--data_path", default=None, type=str, required=True, help="data file path")
parser.add_argument("--save_path", default=None, type=str, required=True, help="stats results save file path")
parser.add_argument("--thread", default=1, type=int, required=True, help="thread num")
args = parser.parse_args()


group_names = [
    "300~500",
    "500~800",
    "800~1000",
    "1000~3000",
    "3000~5000",
    "5000+"
]


def load_data(data_filepath):
    if not os.path.exists("../data/inference_time/classic"):
        os.makedirs("../data/inference_time/classic")
    selected_sequence_groups = []
    for _ in range(len(group_names)):
        selected_sequence_groups.append([])
    with open(data_filepath, "r") as rfp:
        reader = csv.reader(rfp)
        cnt = 0
        for row in reader:
            cnt += 1
            # 去掉表头
            if cnt == 1:
                continue
            # 每一行是一个样本
            seq_id, seq, label, group_idx = row[0], row[1], row[2], int(row[3])
            selected_sequence_groups[group_idx].append(row)
            write_fasta("../data/inference_time/classic/%s.fa" % seq_id, [[seq_id, seq]])
    # add one to load model(增加一个样本到第0组，也就是第0组第0个样本用来加载模型的，不计入时间）
    selected_sequence_groups[0].append(selected_sequence_groups[0][0])
    print("all num: ", [len(item) for item in selected_sequence_groups])
    return selected_sequence_groups


def cmd(method, filenpath, thread):
    if method == "diamond":
        cmd_str = "../tools/classic/diamond blastp -q %s -d ../tools/classic/RdRp20230126.dmnd -o %s.out -e 1E-3 -k 1 -p %d -f 6 qseqid sseqid qlen length pident evalue" % (filenpath,  filenpath, thread)
    elif method == "hmmscan":
        cmd_str = "hmmscan -E 10 --domE 10 --cpu %d --noali --acc --notextw --domtblout %s.hmmscan ../tools/classic/all_known_RdRP.hmm %s" % (thread, filenpath,  filenpath)
    elif method == "palmscan":
        cmd_str = "../tools/classic/palmscan -search_pp %s -rdrp -ppout %s.output -threads %d" % (filenpath,  filenpath, thread)
    else:
        raise Exception("Not support method=%s" % method)
    p = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    p.wait()


def inference_time_stats():
    stats_info = []
    selected_sequence_groups = load_data(args.data_path)
    use_time = []
    for idx1, item in enumerate(selected_sequence_groups):
        if 0 <= args.group_idx < idx1:
            continue
        use_time.append([])
        for idx2, row in enumerate(item):
            protein_id, seq, label = row[0], row[1], row[2]
            # row = [protein_id, seq]
            # Step 3: prediction
            begin = time.time()
            cmd(args.method, "../data/inference_time/classic/%s.fa" % protein_id, args.thread)
            end = time.time()
            # idx = 0, the spend time includes the esmfold model loading time, so remove it
            if idx1 == 0 and idx2 == 0:
                continue
            use_time[-1].append([protein_id, seq, label, idx1, end - begin])
        if len(use_time[-1]) > 0:
            s = "group name(seq length range): %s, num samples: %d, avg-time(s): %f, max-time(s): %f, min-time(s): %f" % (group_names[idx1],
                                                                                                                          len(use_time[-1]),
                     sum([v[-1] for v in use_time[-1]])/len(use_time[-1]),
                     max([v[-1] for v in use_time[-1]]),
                     min([v[-1] for v in use_time[-1]]))
            stats_info.append(s)
    for v in use_time:
        print(len(v))
    for s in stats_info:
        print(s)
    with open(args.save_path, "w") as wfp:
        writer = csv.writer(wfp)
        writer.writerow(["protein_id", "seq", "label", "group_idx", "use_time(s)"])
        for item1 in use_time:
            for item2 in item1:
                writer.writerow(item2)


if __name__ == "__main__":
    inference_time_stats()
    '''
    cd LucaProtApp/algorithms/   
    python inference_time_stats_classic.py \
        --data_path ../data/inference_time/all_selected.csv \
        --save_path ../data/inference_time/all_selected_time_stats_classic_diamond.csv \
        --group_idx -1 \
        --method diamond \
        --thread 1
    
    python inference_time_stats_classic.py \
        --data_path ../data/inference_time/all_selected.csv \
        --save_path ../data/inference_time/all_selected_time_stats_classic_hmmscan.csv \
        --group_idx -1 \
        --method hmmscan \
        --thread 1
    
    python inference_time_stats_classic.py \
        --data_path ../data/inference_time/all_selected.csv \
        --save_path ../data/inference_time/all_selected_time_stats_classic_palmscan.csv \
        --group_idx -1 \
        --method palmscan \
        --thread 1
    '''